@@ -42,8 +42,7 @@ RS_XML(HtmlParseTree)(USER_OBJECT_ fileName, USER_OBJECT_ converterFunctions,
     name = CHARACTER_DATA(fileName)[0];
 #endif
     if(!isURLDoc && (name == NULL || stat(name, &tmp_stat) < 0)) {
-      PROBLEM "Can't find file %s", CHAR_DEREF(STRING_ELT(fileName, 0))
-      ERROR;
+	Rf_error("Can't find file %s", CHAR_DEREF(STRING_ELT(fileName, 0)) );
     }
   } else {
      name = strdup(CHAR_DEREF(STRING_ELT(fileName, 0)));
@@ -51,12 +50,6 @@ RS_XML(HtmlParseTree)(USER_OBJECT_ fileName, USER_OBJECT_ converterFunctions,
   }
 
 
-#if 0
-    /* If one wants entities expanded directly and to appear as text.  */
-  if(LOGICAL_DATA(replaceEntities)[0])
-    xmlSubstituteEntitiesDefault(1);   
-#endif
-
   if(asTextBuffer) {
    doc = htmlParseDoc(CHAR_TO_XMLCHAR(name), NULL);
    if(doc != NULL) {
@@ -69,8 +62,7 @@ RS_XML(HtmlParseTree)(USER_OBJECT_ fileName, USER_OBJECT_ converterFunctions,
   if(doc == NULL) {
     if(freeName && name)
         free((char *) name);
-    PROBLEM "error in creating parser for %s", name
-    ERROR;
+    Rf_error("error in creating parser for %s", name);
   }
 
   PROTECT(rdoc = RS_XML(convertXMLDoc)(name, doc, converterFunctions, &parserSettings));
@@ -111,14 +103,8 @@ RS_XML_dumpHTMLDoc(USER_OBJECT_ r_node, USER_OBJECT_ format, USER_OBJECT_ r_enco
     xmlOutputBufferPtr buf;
     xmlBufferPtr xbuf;
 
-    int oldIndent;
-
-    oldIndent = xmlIndentTreeOutput;
-
     node = (xmlDocPtr) R_ExternalPtrAddr(r_node);
 
-    xmlIndentTreeOutput =  LOGICAL(indent)[0];
-
 #if ADD_XML_OUTPUT_BUFFER_CODE
     if(Rf_length(outFile)) {
        htmlSaveFile(CHAR_DEREF(STRING_ELT(outFile, 0)), node);
@@ -140,16 +126,17 @@ RS_XML_dumpHTMLDoc(USER_OBJECT_ r_node, USER_OBJECT_ format, USER_OBJECT_ r_enco
 
     htmlDocContentDumpFormatOutput(buf, node, encoding, INTEGER(format)[0]);
     xmlOutputBufferFlush(buf);
-    xmlIndentTreeOutput = oldIndent;
 
-    if(xbuf->use > 0) {
+    if(xmlBufferLength(xbuf) > 0) {
         /*XXX this const char * in CHARSXP means we have to make multiple copies. */
 #if 0
 	char *rbuf = R_alloc(sizeof(char) * (xbuf->use + 1));
 	memcpy(rbuf, xbuf->content, xbuf->use + 1);
 	PROTECT(tmp = mkChar(rbuf));
 #endif
-	ans = ScalarString(mkChar(xbuf->content));
+	// ans = ScalarString(mkChar(xbuf->content));
+	DECL_ENCODING_FROM_DOC(node)
+	ans = ScalarString(ENC_COPY_TO_USER_STRING(XMLCHAR_TO_CHAR(xmlBufferContent(xbuf))));
     } else
       ans = NEW_CHARACTER(1);
 
